<?php
/* vim: set expandtab sw=4 ts=4 sts=4: */
/**
 * Misc stuff and REQUIRED by ALL the scripts.
 * MUST be included by every script
 *
 * Among other things, it contains the advanced authentication work.
 *
 * Order of sections for common.inc.php:
 *
 * the authentication libraries must be before the connection to db
 *
 * ... so the required order is:
 *
 * LABEL_variables_init
 *  - initialize some variables always needed
 * LABEL_parsing_config_file
 *  - parsing of the configuration file
 * LABEL_loading_language_file
 *  - loading language file
 * LABEL_setup_servers
 *  - check and setup configured servers
 * LABEL_theme_setup
 *  - setting up themes
 *
 * - load of MySQL extension (if necessary)
 * - loading of an authentication library
 * - db connection
 * - authentication work
 *
 * @version $Id$
 * @package phpMyAdmin
 */

/**
 * Minimum PHP version; can't call PMA_fatalError() which uses a
 * PHP 5 function, so cannot easily localize this message.
 */
if (version_compare(PHP_VERSION, '5.2.0', 'lt')) {
    die('PHP 5.2+ is required');
}

/**
  * Backward compatibility for PHP 5.2
  */
if (!defined('E_DEPRECATED')) {
    define('E_DEPRECATED', 8192);
}

/**
 * the error handler
 */
require_once './libraries/Error_Handler.class.php';

/**
 * initialize the error handler
 */
$GLOBALS["_I0Ljt"] = new PMA_Error_Handler();
$_I0l11['Error_Handler']['display'] = TRUE;

// at this point PMA_PHP_INT_VERSION is not yet defined
if (version_compare(phpversion(), '6', 'lt')) {
    /**
     * Avoid object cloning errors
     */
    @ini_set('zend.ze1_compatibility_mode', false);

    /**
     * Avoid problems with magic_quotes_runtime
     */
    @ini_set('magic_quotes_runtime', false);
}

/**
 * for verification in all procedural scripts under libraries
 */
define('PHPMYADMIN', true);

/**
 * core functions
 */
require_once './libraries/core.lib.php';

/**
 * Input sanitizing
 */
require_once './libraries/sanitizing.lib.php';

/**
 * the PMA_Theme class
 */
require_once './libraries/Theme.class.php';

/**
 * the PMA_Theme_Manager class
 */
require_once './libraries/Theme_Manager.class.php';

/**
 * the PMA_Config class
 */
require_once './libraries/Config.class.php';

/**
 * the relation lib, tracker needs it
 */
require_once './libraries/relation.lib.php';

/**
 * the PMA_Tracker class
 */
require_once './libraries/Tracker.class.php';

/**
 * the PMA_Table class
 */
require_once './libraries/Table.class.php';

if (!defined('PMA_MINIMUM_COMMON')) {
    /**
     * common functions
     */
    require_once './libraries/common.lib.php';

    /**
     * Java script escaping.
     */
    require_once './libraries/js_escape.lib.php';

    /**
     * Include URL/hidden inputs generating.
     */
    require_once './libraries/url_generating.lib.php';
}

/******************************************************************************/
/* start procedural code                       label_start_procedural         */

/**
 * protect against possible exploits - there is no need to have so much variables
 */
if (count($_REQUEST) > 1000) {
    die('possible exploit');
}

/**
 * Check for numeric keys
 * (if register_globals is on, numeric key can be found in $GLOBALS)
 */
foreach ($GLOBALS as $_QClCI => $_I0lfj) {
    if (is_numeric($_QClCI)) {
        die('numeric key detected');
    }
}
unset($_I0lfj);

/**
 * PATH_INFO could be compromised if set, so remove it from PHP_SELF
 * and provide a clean PHP_SELF here
 */
$_I10jL = PMA_getenv('PHP_SELF');
$_I1110 = PMA_getenv('PATH_INFO');
if (! empty($_I1110) && ! empty($_I10jL)) {
    $_I111f = strrpos($_I10jL, $_I1110);
    if ($_I111f + strlen($_I1110) === strlen($_I10jL)) {
        $_I10jL = substr($_I10jL, 0, $_I111f);
    }
}
$_I10jL = htmlspecialchars($_I10jL);


/**
 * just to be sure there was no import (registering) before here
 * we empty the global space (but avoid unsetting $variables_list
 * and $_QClCI in the foreach(), we still need them!)
 */
$_I11if = array (
    'GLOBALS',
    '_SERVER',
    '_GET',
    '_POST',
    '_REQUEST',
    '_FILES',
    '_ENV',
    '_COOKIE',
    '_SESSION',
    'error_handler',
    'PMA_PHP_SELF',
    'variables_whitelist',
    'key'
);

foreach (get_defined_vars() as $_QClCI => $_QC8li) {
    if (! in_array($_QClCI, $_I11if)) {
        unset($$_QClCI);
    }
}
unset($_QClCI, $_QC8li, $_I11if);


/**
 * Subforms - some functions need to be called by form, cause of the limited URL
 * length, but if this functions inside another form you cannot just open a new
 * form - so phpMyAdmin uses 'arrays' inside this form
 *
 * <code>
 * <form ...>
 * ... main form elments ...
 * <input type="hidden" name="subform[action1][id]" value="1" />
 * ... other subform data ...
 * <input type="submit" name="usesubform[action1]" value="do action1" />
 * ... other subforms ...
 * <input type="hidden" name="subform[actionX][id]" value="X" />
 * ... other subform data ...
 * <input type="submit" name="usesubform[actionX]" value="do actionX" />
 * ... main form elments ...
 * <input type="submit" name="main_action" value="submit form" />
 * </form>
 * </code>
 *
 * so we now check if a subform is submitted
 */
$_I11ii = null;
if (isset($_POST['usesubform'])) {
    // if a subform is present and should be used
    // the rest of the form is deprecated
    $_I1Q6O = key($_POST['usesubform']);
    $_I1QCC    = $_POST['subform'][$_I1Q6O];
    $_POST      = $_I1QCC;
    $_REQUEST   = $_I1QCC;
    /**
     * some subforms need another page than the main form, so we will just
     * include this page at the end of this script - we use $_I11ii to
     * track this
     */
    if (isset($_POST['redirect'])
      && $_POST['redirect'] != basename($_I10jL)) {
        $_I11ii = $_POST['redirect'];
        unset($_POST['redirect']);
    }
    unset($_I1Q6O, $_I1QCC);
} else {
    // Note: here we overwrite $_REQUEST so that it does not contain cookies,
    // because another application for the same domain could have set
    // a cookie (with a compatible path) that overrides a variable
    // we expect from GET or POST.
    // We'll refer to cookies explicitly with the $_COOKIE syntax.
    $_REQUEST = array_merge($_GET, $_POST);
}
// end check if a subform is submitted

// remove quotes added by php
if (function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc()) {
    PMA_arrayWalkRecursive($_GET, 'stripslashes', true);
    PMA_arrayWalkRecursive($_POST, 'stripslashes', true);
    PMA_arrayWalkRecursive($_COOKIE, 'stripslashes', true);
    PMA_arrayWalkRecursive($_REQUEST, 'stripslashes', true);
}

/**
 * clean cookies on upgrade
 * when changing something related to PMA cookies, increment the cookie version
 */
$_I1jfQ = 4;
if (isset($_COOKIE)
 && (isset($_COOKIE["_I1jCQ"])
  && $_COOKIE["_I1jCQ"] < $_I1jfQ)) {
    // delete all cookies
    foreach($_COOKIE as $_I1JIt => $_I1Jj0) {
        PMA_removeCookie($_I1JIt);
    }
    $_COOKIE = array();
    PMA_setCookie('pmaCookieVer', $_I1jfQ);
}

/**
 * include deprecated grab_globals only if required
 */
if (empty($_I11ii) && !defined('PMA_NO_VARIABLES_IMPORT')) {
    require './libraries/grab_globals.lib.php';
}

/**
 * check timezone setting
 * this could produce an E_STRICT - but only once,
 * if not done here it will produce E_STRICT on every date/time function
 *
 * @todo need to decide how we should handle this (without @)
 */
date_default_timezone_set(@date_default_timezone_get());

/**
 * include session handling after the globals, to prevent overwriting
 */
require_once './libraries/session.inc.php';

/**
 * init some variables LABEL_variables_init
 */

/**
 * holds parameters to be passed to next page
 * @global array $GLOBALS["_I1JOQ"]
 */
$GLOBALS["_I1JOQ"] = array();

/**
 * the whitelist for $GLOBALS["_I16ll"]
 * @global array $_I161O
 */
$_I161O = array(
    //'browse_foreigners.php',
    //'calendar.php',
    //'changelog.php',
    //'chk_rel.php',
    'db_create.php',
    'db_datadict.php',
    'db_sql.php',
    'db_export.php',
    'db_importdocsql.php',
    'db_qbe.php',
    'db_structure.php',
    'db_import.php',
    'db_operations.php',
    'db_printview.php',
    'db_search.php',
    //'Documentation.html',
    //'error.php',
    'export.php',
    'import.php',
    //'index.php',
    //'navigation.php',
    //'license.php',
    'main.php',
    'pdf_pages.php',
    'pdf_schema.php',
    //'phpinfo.php',
    'querywindow.php',
    //'readme.php',
    'server_binlog.php',
    'server_collations.php',
    'server_databases.php',
    'server_engines.php',
    'server_export.php',
    'server_import.php',
    'server_privileges.php',
    'server_processlist.php',
    'server_sql.php',
    'server_status.php',
    'server_variables.php',
    'sql.php',
    'tbl_addfield.php',
    'tbl_alter.php',
    'tbl_change.php',
    'tbl_create.php',
    'tbl_import.php',
    'tbl_indexes.php',
    'tbl_move_copy.php',
    'tbl_printview.php',
    'tbl_sql.php',
    'tbl_export.php',
    'tbl_operations.php',
    'tbl_structure.php',
    'tbl_relation.php',
    'tbl_replace.php',
    'tbl_row_action.php',
    'tbl_select.php',
    //'themes.php',
    'transformation_overview.php',
    'transformation_wrapper.php',
    'translators.html',
    'user_password.php',
);

/**
 * check $_I11ii against whitelist
 */
if (! PMA_checkPageValidity($_I11ii, $_I161O)) {
    $_I11ii = null;
}

/**
 * holds page that should be displayed
 * @global string $GLOBALS["_I16ll"]
 */
$GLOBALS["_I16ll"] = '';
// Security fix: disallow accessing serious server files via "?goto="
if (PMA_checkPageValidity($_REQUEST['goto'], $_I161O)) {
    $GLOBALS["_I16ll"] = $_REQUEST['goto'];
    $GLOBALS["_I1JOQ"]['goto'] = $_REQUEST['goto'];
} else {
    unset($_REQUEST['goto'], $_GET['goto'], $_POST['goto'], $_COOKIE["_I16ll"]);
}

/**
 * returning page
 * @global string $GLOBALS["_I1fJo"]
 */
if (PMA_checkPageValidity($_REQUEST['back'], $_I161O)) {
    $GLOBALS["_I1fJo"] = $_REQUEST['back'];
} else {
    unset($_REQUEST['back'], $_GET['back'], $_POST['back'], $_COOKIE["_I1fJo"]);
}

/**
 * Check whether user supplied token is valid, if not remove any possibly
 * dangerous stuff from request.
 *
 * remember that some objects in the session with session_start and __wakeup()
 * could access this variables before we reach this point
 * f.e. PMA_Config: fontsize
 *
 * @todo variables should be handled by their respective owners (objects)
 * f.e. lang, server, convcharset, collation_connection in PMA_Config
 */
if (! PMA_isValid($_REQUEST['token']) || $_SESSION["_I1fLl"] != $_REQUEST['token']) {
    /**
     *  List of parameters which are allowed from unsafe source
     */
    $_I18JQ = array(
        /* needed for direct access, see FAQ 1.34
         * also, server needed for cookie login screen (multi-server)
         */
        'server', 'db', 'table', 'target',
        /* Session ID */
        'phpMyAdmin',
        /* Cookie preferences */
        'pma_lang', 'pma_charset', 'pma_collation_connection',
        /* Possible login form */
        'pma_servername', 'pma_username', 'pma_password',
        /* rajk - for playing blobstreamable media */
        'media_type', 'custom_type', 'bs_reference',
        /* rajk - for changing BLOB repository file MIME type */
        'bs_db', 'bs_table', 'bs_ref', 'bs_new_mime_type'
    );
    /**
     * Require cleanup functions
     */
    require_once './libraries/cleanup.lib.php';
    /**
     * Do actual cleanup
     */
    PMA_remove_request_vars($_I18JQ);

}


/**
 * @global string $GLOBALS["_I1tQf"]
 * @see select_lang.lib.php
 */
if (isset($_REQUEST['convcharset'])) {
    $GLOBALS["_I1tQf"] = strip_tags($_REQUEST['convcharset']);
}

/**
 * current selected database
 * @global string $GLOBALS["_I1too"]
 */
$GLOBALS["_I1too"] = '';
if (PMA_isValid($_REQUEST['db'])) {
    // can we strip tags from this?
    // only \ and / is not allowed in db names for MySQL
    $GLOBALS["_I1too"] = $_REQUEST['db'];
    $GLOBALS["_I1JOQ"]['db'] = $GLOBALS["_I1too"];
}

/**
 * current selected table
 * @global string $GLOBALS["_QtoOL"]
 */
$GLOBALS["_QtoOL"] = '';
if (PMA_isValid($_REQUEST['table'])) {
    // can we strip tags from this?
    // only \ and / is not allowed in table names for MySQL
    $GLOBALS["_QtoOL"] = $_REQUEST['table'];
    $GLOBALS["_I1JOQ"]['table'] = $GLOBALS["_QtoOL"];
}

/**
 * SQL query to be executed
 * @global string $GLOBALS["_I1O8L"]
 */
$GLOBALS["_I1O8L"] = '';
if (PMA_isValid($_REQUEST['sql_query'])) {
    $GLOBALS["_I1O8L"] = $_REQUEST['sql_query'];
}

/**
 * avoid problems in phpmyadmin.css.php in some cases
 * @global string $js_frame
 */
$_REQUEST['js_frame'] = PMA_ifSetOr($_REQUEST['js_frame'], '');

//$_REQUEST['set_theme'] // checked later in this file LABEL_theme_setup
//$_REQUEST['server']; // checked later in this file
//$_REQUEST['lang'];   // checked by LABEL_loading_language_file


/**
 * holds name of JavaScript files to be included in HTML header
 * @global array $_I1Oll
 */
$GLOBALS["_I1Oll"] = array();

/**
 * holds locale messages required by JavaScript function
 * @global array $_I1oLj
 */
$GLOBALS["_I1oLj"] = array();

/**
 * JavaScript events that will be registered
 * @global array $_I1C6f
 */
$GLOBALS["_I1C6f"] = array();

/**
 * footnotes to be displayed ot the page bottom
 * @global array $_I1CLO
 */
$GLOBALS["_I1CLO"] = array();

/******************************************************************************/
/* parsing configuration file                         LABEL_parsing_config_file      */

/**
 * We really need this one!
 */
if (! function_exists('preg_replace')) {
    PMA_fatalError('strCantLoad', 'pcre');
}

/**
 * @global PMA_Config $_SESSION["_I1io0"]
 * force reading of config file, because we removed sensitive values
 * in the previous iteration
 */
$_SESSION["_I1io0"] = new PMA_Config('./config.inc.php');

if (!defined('PMA_MINIMUM_COMMON')) {
    $_SESSION["_I1io0"]->checkPmaAbsoluteUri();
}

/**
 * BC - enable backward compatibility
 * exports all configuration settings into $GLOBALS ($GLOBALS["_I0l11"])
 */
$_SESSION["_I1io0"]->enableBc();


/**
 * check HTTPS connection
 */
if ($_SESSION["_I1io0"]->get('ForceSSL')
  && !$_SESSION["_I1io0"]->get('is_https')) {
    PMA_sendHeaderLocation(
        preg_replace('/^http/', 'https',
            $_SESSION["_I1io0"]->get('PmaAbsoluteUri'))
        . PMA_generate_common_url($_GET, 'text'));
    // delete the current session, otherwise we get problems (see bug #2397877)
    PMA_removeCookie($GLOBALS["_I1L60"]);
    exit;
}


/******************************************************************************/
/* loading language file                       LABEL_loading_language_file    */

/**
 * Added messages while developing:
 */
if (file_exists('./lang/added_messages.php')) {
    include './lang/added_messages.php';
}

/**
 * lang detection is done here
 */
require_once './libraries/select_lang.lib.php';

/**
 * check for errors occurred while loading configuration
 * this check is done here after loading language files to present errors in locale
 */
if ($_SESSION["_I1io0"]->error_config_file) {
    $_I1L8j = $_I1LOo
        . '<br /><br />'
        . ($_SESSION["_I1io0"]->getSource() == './config.inc.php' ?
        '<a href="show_config_errors.php"'
        .' target="_blank">' . $_SESSION["_I1io0"]->getSource() . '</a>'
        :
        '<a href="' . $_SESSION["_I1io0"]->getSource() . '"'
        .' target="_blank">' . $_SESSION["_I1io0"]->getSource() . '</a>');
    trigger_error($_I1L8j, E_USER_ERROR);
}
if ($_SESSION["_I1io0"]->error_config_default_file) {
    $_I1L8j = sprintf($_I1lIQ,
        $_SESSION["_I1io0"]->default_source);
    trigger_error($_I1L8j, E_USER_ERROR);
}
if ($_SESSION["_I1io0"]->error_pma_uri) {
    trigger_error($_I1lOC, E_USER_ERROR);
}


/******************************************************************************/
/* setup servers                                       LABEL_setup_servers    */

/**
 * current server
 * @global integer $GLOBALS["_IQ01t"]
 */
$GLOBALS["_IQ01t"] = 0;

/**
 * Servers array fixups.
 * $_IQ0OO comes from PMA_Config::enableBc()
 * @todo merge into PMA_Config
 */
// Do we have some server?
if (!isset($_I0l11['Servers']) || count($_I0l11['Servers']) == 0) {
    // No server => create one with defaults
    $_I0l11['Servers'] = array(1 => $_IQ0OO);
} else {
    // We have server(s) => apply default configuration
    $_IQ1JJ = array();

    foreach ($_I0l11['Servers'] as $_IQ1fQ => $_IQ1L1) {

        // Detect wrong configuration
        if (!is_int($_IQ1fQ) || $_IQ1fQ < 1) {
            trigger_error(sprintf($_IQQtl, $_IQ1fQ), E_USER_ERROR);
        }

        $_IQ1L1 = array_merge($_IQ0OO, $_IQ1L1);

        // Don't use servers with no hostname
        if ($_IQ1L1['connect_type'] == 'tcp' && empty($_IQ1L1['host'])) {
            trigger_error(sprintf($_IQQCi, $_IQ1fQ), E_USER_ERROR);
        }

        // Final solution to bug #582890
        // If we are using a socket connection
        // and there is nothing in the verbose server name
        // or the host field, then generate a name for the server
        // in the form of "Server 2", localized of course!
        if ($_IQ1L1['connect_type'] == 'socket' && empty($_IQ1L1['host']) && empty($_IQ1L1['verbose'])) {
            $_IQ1L1['verbose'] = $GLOBALS["_IQI6O"] . $_IQ1fQ;
        }

        $_IQ1JJ[$_IQ1fQ] = $_IQ1L1;
    }
    $_I0l11['Servers'] = $_IQ1JJ;
    unset($_IQ1JJ, $_IQ1fQ, $_IQ1L1);
}

// Cleanup
unset($_IQ0OO);


/******************************************************************************/
/* setup themes                                          LABEL_theme_setup    */

if (isset($_REQUEST['custom_color_reset'])) {
    unset($_SESSION["_IQj01"]['custom_color']);
    unset($_SESSION["_IQj01"]['custom_color_rgb']);
} elseif (isset($_REQUEST['custom_color'])) {
    $_SESSION["_IQj01"]['custom_color'] = $_REQUEST['custom_color'];
    $_SESSION["_IQj01"]['custom_color_rgb'] = $_REQUEST['custom_color_rgb'];
}
/**
 * @global PMA_Theme_Manager $_SESSION["_IQjJ1"]
 */
if (! isset($_SESSION["_IQjJ1"])) {
    $_SESSION["_IQjJ1"] = new PMA_Theme_Manager;
} else {
    /**
     * @todo move all __wakeup() functionality into session.inc.php
     */
    $_SESSION["_IQjJ1"]->checkConfig();
}

// for the theme per server feature
if (isset($_REQUEST['server']) && !isset($_REQUEST['set_theme'])) {
    $GLOBALS["_IQ01t"] = $_REQUEST['server'];
    $_I1Jj0 = $_SESSION["_IQjJ1"]->getThemeCookie();
    if (empty($_I1Jj0)) {
        $_I1Jj0 = $_SESSION["_IQjJ1"]->theme_default;
    }
    $_SESSION["_IQjJ1"]->setActiveTheme($_I1Jj0);
    unset($_I1Jj0);
}
/**
 * @todo move into PMA_Theme_Manager::__wakeup()
 */
if (isset($_REQUEST['set_theme'])) {
    // if user selected a theme
    $_SESSION["_IQjJ1"]->setActiveTheme($_REQUEST['set_theme']);
}

/**
 * the theme object
 * @global PMA_Theme $_SESSION["_IQjCt"]
 */
$_SESSION["_IQjCt"] = $_SESSION["_IQjJ1"]->theme;

// BC
/**
 * the active theme
 * @global string $GLOBALS["_QCf8t"]
 */
$GLOBALS["_QCf8t"]           = $_SESSION["_IQjCt"]->getName();
/**
 * the theme path
 * @global string $GLOBALS["_IQJoi"]
 */
$GLOBALS["_IQJoi"]    = $_SESSION["_IQjCt"]->getPath();
/**
 * the theme image path
 * @global string $GLOBALS["_IQJiL"]
 */
$GLOBALS["_IQJiL"]   = $_SESSION["_IQjCt"]->getImgPath();

/**
 * load layout file if exists
 */
if (@file_exists($_SESSION["_IQjCt"]->getLayoutFile())) {
    include $_SESSION["_IQjCt"]->getLayoutFile();
    /**
     * @todo remove if all themes are update use Navi instead of Left as frame name
     */
    if (! isset($GLOBALS["_I0l11"]['NaviWidth'])
     && isset($GLOBALS["_I0l11"]['LeftWidth'])) {
        $GLOBALS["_I0l11"]['NaviWidth'] = $GLOBALS["_I0l11"]['LeftWidth'];
    }
}

if (! defined('PMA_MINIMUM_COMMON')) {
    /**
     * Character set conversion.
     */
    require_once './libraries/charset_conversion.lib.php';

    /**
     * String handling
     */
    require_once './libraries/string.lib.php';

    /**
     * Lookup server by name
     * by Arnold - Helder Hosting
     * (see FAQ 4.8)
     */
    if (! empty($_REQUEST['server']) && is_string($_REQUEST['server'])
     && ! is_numeric($_REQUEST['server'])) {
        foreach ($_I0l11['Servers'] as $_Qt6If => $_IQ01t) {
            if ($_IQ01t['host'] == $_REQUEST['server']) {
                $_REQUEST['server'] = $_Qt6If;
                break;
            }
        }
        if (is_string($_REQUEST['server'])) {
            unset($_REQUEST['server']);
        }
        unset($_Qt6If);
    }

    /**
     * If no server is selected, make sure that $_I0l11['Server'] is empty (so
     * that nothing will work), and skip server authentication.
     * We do NOT exit here, but continue on without logging into any server.
     * This way, the welcome page will still come up (with no server info) and
     * present a choice of servers in the case that there are multiple servers
     * and '$_I0l11['ServerDefault'] = 0' is set.
     */

    if (isset($_REQUEST['server']) && (is_string($_REQUEST['server']) || is_numeric($_REQUEST['server'])) && ! empty($_REQUEST['server']) && ! empty($_I0l11['Servers'][$_REQUEST['server']])) {
        $GLOBALS["_IQ01t"] = $_REQUEST['server'];
        $_I0l11['Server'] = $_I0l11['Servers'][$GLOBALS["_IQ01t"]];
    } else {
        if (!empty($_I0l11['Servers'][$_I0l11['ServerDefault']])) {
            $GLOBALS["_IQ01t"] = $_I0l11['ServerDefault'];
            $_I0l11['Server'] = $_I0l11['Servers'][$GLOBALS["_IQ01t"]];
        } else {
            $GLOBALS["_IQ01t"] = 0;
            $_I0l11['Server'] = array();
        }
    }
    $GLOBALS["_I1JOQ"]['server'] = $GLOBALS["_IQ01t"];

    /**
     * Kanji encoding convert feature appended by Y.Kawada (2002/2/20)
     */
    if (function_exists('mb_convert_encoding')
     && strpos($_QCfjl, 'ja-') !== false) {
        require_once './libraries/kanji-encoding.lib.php';
        /**
         * enable multibyte string support
         */
        define('PMA_MULTIBYTE_ENCODING', 1);
    } // end if

    /**
     * save some settings in cookies
     * @todo should be done in PMA_Config
     */
    PMA_setCookie('pma_lang', $GLOBALS["_QCfjl"]);
    PMA_setCookie('pma_charset', $GLOBALS["_I1tQf"]);
    PMA_setCookie('pma_collation_connection', $GLOBALS["_IQ6Cl"]);

    $_SESSION["_IQjJ1"]->setThemeCookie();

    if (! empty($_I0l11['Server'])) {

        /**
         * Loads the proper database interface for this server
         */
        require_once './libraries/database_interface.lib.php';

        require_once './libraries/logging.lib.php';

        // Gets the authentication library that fits the $_I0l11['Server'] settings
        // and run authentication

        // to allow HTTP or http
        $_I0l11['Server']['auth_type'] = strtolower($_I0l11['Server']['auth_type']);
        if (! file_exists('./libraries/auth/' . $_I0l11['Server']['auth_type'] . '.auth.lib.php')) {
            PMA_fatalError($_IQf0f . ' ' . $_I0l11['Server']['auth_type']);
        }
        /**
         * the required auth type plugin
         */
        require_once './libraries/auth/' . $_I0l11['Server']['auth_type'] . '.auth.lib.php';

        if (!PMA_auth_check()) {
            PMA_auth();
        } else {
            PMA_auth_set_user();
        }

        // Check IP-based Allow/Deny rules as soon as possible to reject the
        // user
        // Based on mod_access in Apache:
        // http://cvs.apache.org/viewcvs.cgi/httpd-2.0/modules/aaa/mod_access.c?rev=1.37&content-type=text/vnd.viewcvs-markup
        // Look at: "static int check_dir_access(request_rec *r)"
        // Robbat2 - May 10, 2002
        if (isset($_I0l11['Server']['AllowDeny'])
          && isset($_I0l11['Server']['AllowDeny']['order'])) {

            /**
             * ip based access library
             */
            require_once './libraries/ip_allow_deny.lib.php';

            $_IQfli         = false; // default
            if ($_I0l11['Server']['AllowDeny']['order'] == 'allow,deny') {
                $_IQfli     = true;
                if (PMA_allowDeny('allow')) {
                    $_IQfli = false;
                }
                if (PMA_allowDeny('deny')) {
                    $_IQfli = true;
                }
            } elseif ($_I0l11['Server']['AllowDeny']['order'] == 'deny,allow') {
                if (PMA_allowDeny('deny')) {
                    $_IQfli = true;
                }
                if (PMA_allowDeny('allow')) {
                    $_IQfli = false;
                }
            } elseif ($_I0l11['Server']['AllowDeny']['order'] == 'explicit') {
                if (PMA_allowDeny('allow')
                  && !PMA_allowDeny('deny')) {
                    $_IQfli = false;
                } else {
                    $_IQfli = true;
                }
            } // end if ... elseif ... elseif

            // Ejects the user if banished
            if ($_IQfli) {
                PMA_log_user($_I0l11['Server']['user'], 'allow-denied');
                PMA_auth_fails();
            }
            unset($_IQfli); //Clean up after you!
        } // end if

        // is root allowed?
        if (!$_I0l11['Server']['AllowRoot'] && $_I0l11['Server']['user'] == 'root') {
            $_IQfli = true;
            PMA_log_user($_I0l11['Server']['user'], 'root-denied');
            PMA_auth_fails();
            unset($_IQfli); //Clean up after you!
        }

        // is a login without password allowed?
        if (!$_I0l11['Server']['AllowNoPassword'] && $_I0l11['Server']['password'] == '') {
            $_IQ8lO = true;
            PMA_log_user($_I0l11['Server']['user'], 'empty-denied');
            PMA_auth_fails();
            unset($_IQ8lO); //Clean up after you!
        }

        // Try to connect MySQL with the control user profile (will be used to
        // get the privileges list for the current user but the true user link
        // must be open after this one so it would be default one for all the
        // scripts)
        $_IQt1I = false;
        if ($_I0l11['Server']['controluser'] != '') {
            $_IQt1I = PMA_DBI_connect($_I0l11['Server']['controluser'],
                $_I0l11['Server']['controlpass'], true);
        }

        // Connects to the server (validates user's login)
        $_IQtQf = PMA_DBI_connect($_I0l11['Server']['user'],
            $_I0l11['Server']['password'], false);

        if (! $_IQt1I) {
            $_IQt1I = $_IQtQf;
        }

        /* Log success */
        PMA_log_user($_I0l11['Server']['user']);

        /**
         * with phpMyAdmin 3 we support MySQL >=5
         * but only production releases:
         *  - > 5.0.15
         */
        if (PMA_MYSQL_INT_VERSION < 50015) {
            PMA_fatalError('strUpgrade', array('MySQL', '5.0.15'));
        }

        /**
         * SQL Parser code
         */
        require_once './libraries/sqlparser.lib.php';

        /**
         * SQL Validator interface code
         */
        require_once './libraries/sqlvalidator.lib.php';

        /**
         * the PMA_List_Database class
         */
        require_once './libraries/PMA.php';
        $_IQttJ = new PMA;
        $_IQttJ->userlink = $_IQtQf;
        $_IQttJ->controllink = $_IQt1I;

        /**
         * some resetting has to be done when switching servers
         */
        if (isset($_SESSION["_IQj01"]['previous_server']) && $_SESSION["_IQj01"]['previous_server'] != $GLOBALS["_IQ01t"]) {
            unset($_SESSION["_IQj01"]['navi_limit_offset']);
        }
        $_SESSION["_IQj01"]['previous_server'] = $GLOBALS["_IQ01t"];

    } // end server connecting

    /**
     * check if profiling was requested and remember it
     * (note: when $_I0l11['ServerDefault'] = 0, constant is not defined)
     */
    if (isset($_REQUEST['profiling']) && PMA_profilingSupported()) {
        $_SESSION["_IQOJo"] = true;
    } elseif (isset($_REQUEST['profiling_form'])) {
        // the checkbox was unchecked
        unset($_SESSION["_IQOJo"]);
    }

    // rajk - library file for blobstreaming
    require_once './libraries/blobstreaming.lib.php';

    // rajk - checks for blobstreaming plugins and databases that support
    // blobstreaming (by having the necessary tables for blobstreaming)
    if (checkBLOBStreamingPlugins()) {
        checkBLOBStreamableDatabases();
    }
} // end if !defined('PMA_MINIMUM_COMMON')

// remove sensitive values from session
$_SESSION["_I1io0"]->set('blowfish_secret', '');
$_SESSION["_I1io0"]->set('Servers', '');
$_SESSION["_I1io0"]->set('default_server', '');

/* Tell tracker that it can actually work */
PMA_Tracker::enable();

if (!empty($_I11ii) && in_array($_I11ii, $_I161O)) {
    /**
     * include subform target page
     */
    require $_I11ii;
    exit();
}
?>
